//Wheelchair motion with simple PID control system
//DO NOT SWITCH ON THE POWER BUTTON ON THE CHASIS IF THE ARDUINO IS POWERED ON WITH USB C!!!

#include <Wire.h>
#include "SparkFun_BMI270_Arduino_Library.h"

// L298N motor driver pins
#define ENA 9   // PWM pin for left motor speed
#define IN1 8   // Left motor direction 1
#define IN2 7   // Left motor direction 2
#define IN3 5   // Right motor direction 1
#define IN4 4   // Right motor direction 2
#define ENB 3  // PWM pin for right motor speed

// PID constants (tune these)
float Kp = 1.5;  // Proportional gain
float Ki = 0.1;  // Integral gain
float Kd = 0.05; // Derivative gain

// PID variables
float error = 0, lastError = 0, integral = 0;
int Direction = 0;
int leftSpeed = 0;
int rightSpeed = 0;
int baseSpeed = 200; // Base motor speed (0-255)

// BMI270 IMU object
BMI270 imu;
uint8_t i2cAddress = BMI2_I2C_PRIM_ADDR; // 0x68

void setup() {
  // Initialize serial for debugging
  Serial.begin(115200);

  // Set motor pins as outputs
  pinMode(ENA, OUTPUT);
  pinMode(ENB, OUTPUT);
  pinMode(IN1, OUTPUT);
  pinMode(IN2, OUTPUT);
  pinMode(IN3, OUTPUT);
  pinMode(IN4, OUTPUT);

  // Initialize I2C
  Wire.begin();

  // Initialize BMI270 (default I2C address is 0x68 or 0x69, depending on SDO pin)
  while(imu.beginI2C(i2cAddress) != BMI2_OK)
    {
        // Not connected, inform user
        Serial.println("Error: BMI270 not connected, check wiring and I2C address!");

        // Wait a bit to see if connection is established
        delay(1000);
    }

    Serial.println("BMI270 connected!");

  // Configure IMU (default settings are usually fine)
  imu.setGyroODR(BMI2_GYR_ODR_100HZ);       // 100 Hz update rate

  // Initialize motor to stop mode
  digitalWrite(IN1, LOW);
  digitalWrite(IN2, LOW);
  digitalWrite(IN3, LOW);
  digitalWrite(IN4, LOW);

  // Set zero Motor Speed 
  analogWrite(ENA, 0);
  analogWrite(ENB, 0);
}

void loop() {
  MoveForward();
  // MoveBackward();
}

//Move Forward
void MoveForward() {
	digitalWrite(IN2, HIGH);
	digitalWrite(IN1, LOW);
	digitalWrite(IN4, HIGH);
	digitalWrite(IN3, LOW);
  Direction = 1;

  //check error to make sure moving in a straight line
  IMU_Control();
}

//Move Backwards
void MoveBackward() {
	digitalWrite(IN2, LOW);
	digitalWrite(IN1, HIGH);
	digitalWrite(IN4, LOW);
	digitalWrite(IN3, HIGH);
  Direction = -1;

  //check error to make sure moving in a straight line
  IMU_Control();
}

//Turn Left
void TurnLeft() {
	digitalWrite(IN2, HIGH);
	digitalWrite(IN1, LOW);
	digitalWrite(IN4, LOW);
	digitalWrite(IN3, LOW);
  Direction = 0;
}

//Turn Right
void TurnRight() {
	digitalWrite(IN2, LOW);
	digitalWrite(IN1, LOW);
	digitalWrite(IN4, HIGH);
	digitalWrite(IN3, LOW);
  Direction = 0;
}

//Stop Moving
void Stop() {
  digitalWrite(IN1, LOW);
	digitalWrite(IN2, LOW);
	digitalWrite(IN3, LOW);
	digitalWrite(IN4, LOW);
  Direction = 0;
}

void IMU_Control() {
  // Update IMU data
  imu.getSensorData();

  // Use gyro Z-axis (yaw) for direction control (degrees per second)
  float gyroZ = imu.data.gyroZ;

  // Calculate error (desired yaw rate = 0 for straight)
  error = 0 - gyroZ;

  // Update integral (dt = 10ms)
  integral += error * 0.01;
  integral = constrain(integral, -50, 50); // Limit windup

  // Calculate derivative
  float derivative = (error - lastError) / 0.01;

  // PID output
  float output = Kp * error + Ki * integral + Kd * derivative;

  // Adjust motor speeds
  if(Direction == 1){ //Moving Forward
    leftSpeed = baseSpeed - output;
    rightSpeed = baseSpeed + output;
  }
  else if(Direction == -1){ //Moving Backward
    leftSpeed = baseSpeed + output;
    rightSpeed = baseSpeed - output;
  }

  // Constrain speeds to valid PWM range
  leftSpeed = constrain(leftSpeed, 0, 255);
  rightSpeed = constrain(rightSpeed, 0, 255);

  // Apply speeds to motors
  analogWrite(ENA, leftSpeed);
  analogWrite(ENB, rightSpeed);

  // Debugging output (optional)
  // Serial.print("GyroZ: "); Serial.print(gyroZ);
  // Serial.print(" | Left: "); Serial.print(leftSpeed);
  // Serial.print(" | Right: "); Serial.println(rightSpeed);

  // Store error for next iteration
  lastError = error;

  // Small delay for timing consistency
  delay(10);
}