#include <WiFiS3.h>

// WiFi credentials
const char ssid[] = "SOTON-IoT";    
const char pass[] = "pUguZXWtBDzl";    
int status = WL_IDLE_STATUS;

// Server settings
WiFiServer server(80);

// Motor control pins
// Motor A - Left side
const int ENA = 10;  // PWM speed control
const int IN1 = 9;   // Direction control
const int IN2 = 8;   // Direction control

// Motor B - Right side
const int ENB = 5;   // PWM speed control
const int IN3 = 7;   // Direction control
const int IN4 = 6;   // Direction control

// Motor speed (0-255)
const int SPEED = 200;
const int TURN_SPEED = 180;

void setup() {
  Serial.begin(9600);
  
  // Initialize motor pins
  pinMode(ENA, OUTPUT);
  pinMode(IN1, OUTPUT);
  pinMode(IN2, OUTPUT);
  pinMode(ENB, OUTPUT);
  pinMode(IN3, OUTPUT);
  pinMode(IN4, OUTPUT);
  
  // Initialize motors to stopped state
  stopMotors();
  
  // Check for WiFi module
  if (WiFi.status() == WL_NO_MODULE) {
    Serial.println("Communication with WiFi module failed!");
    while (true);
  }

  // Check firmware version
  String fv = WiFi.firmwareVersion();
  Serial.print("Firmware version: ");
  Serial.println(fv);

  // Attempt to connect to WiFi network
  while (status != WL_CONNECTED) {
    Serial.print("Attempting to connect to Network: ");
    Serial.println(ssid);
    status = WiFi.begin(ssid, pass);
    delay(10000);
  }

  Serial.println("Connected to WiFi");
  printWiFiStatus();
  
  server.begin();
  Serial.println("EMG-controlled car server started");
}

void loop() {
  WiFiClient client = server.available();
  
  if (client) {
    Serial.println("New client connected");
    String request = "";
    boolean currentLineIsBlank = true;
    
    while (client.connected()) {
      if (client.available()) {
        char c = client.read();
        request += c;
        
        if (c == '\n' && currentLineIsBlank) {
          // Send HTTP response
          client.println("HTTP/1.1 200 OK");
          client.println("Content-Type: text/plain");
          client.println("Connection: close");
          client.println();
          client.println("Command received");
          break;
        }
        if (c == '\n') {
          currentLineIsBlank = true;
        } else if (c != '\r') {
          currentLineIsBlank = false;
        }
      }
    }
    
    // Process the request
    if (request.indexOf("GET /") != -1) {
      processRequest(request);
    }
    
    delay(10);
    client.stop();
    Serial.println("Client disconnected");
  }
}

void processRequest(String request) {
  int start = request.indexOf("GET ") + 4;
  int end = request.indexOf(" HTTP");
  
  if (start != -1 && end != -1) {
    String command = request.substring(start, end);
    Serial.println("Command: " + command);
    
    // Parse command and control motors
    if (command == "/forward") {
      moveForward();
    }
    else if (command == "/backward") {
      moveBackward();
    }
    else if (command == "/left") {
      turnLeft();
    }
    else if (command == "/right") {
      turnRight();
    }
    else if (command == "/stop") {
      stopMotors();
    }
    else if (command == "/ping") {
      // Just for connection testing
      Serial.println("Ping received");
    }
  }
}

void moveForward() {
  Serial.println("Moving forward");
  // Motor A forward
  digitalWrite(IN1, HIGH);
  digitalWrite(IN2, LOW);
  analogWrite(ENA, SPEED);
  
  // Motor B forward
  digitalWrite(IN3, HIGH);
  digitalWrite(IN4, LOW);
  analogWrite(ENB, SPEED);
}

void moveBackward() {
  Serial.println("Moving backward");
  // Motor A backward
  digitalWrite(IN1, LOW);
  digitalWrite(IN2, HIGH);
  analogWrite(ENA, SPEED);
  
  // Motor B backward
  digitalWrite(IN3, LOW);
  digitalWrite(IN4, HIGH);
  analogWrite(ENB, SPEED);
}

void turnLeft() {
  Serial.println("Turning left");
  // Motor A forward faster
  digitalWrite(IN1, HIGH);
  digitalWrite(IN2, LOW);
  analogWrite(ENA, SPEED);
  
  // Motor B slower or reverse for sharper turn
  digitalWrite(IN3, LOW);
  digitalWrite(IN4, HIGH);
  analogWrite(ENB, TURN_SPEED);
}

void turnRight() {
  Serial.println("Turning right");
  // Motor A slower or reverse for sharper turn
  digitalWrite(IN1, LOW);
  digitalWrite(IN2, HIGH);
  analogWrite(ENA, TURN_SPEED);
  
  // Motor B forward faster
  digitalWrite(IN3, HIGH);
  digitalWrite(IN4, LOW);
  analogWrite(ENB, SPEED);
}

void stopMotors() {
  Serial.println("Stopping motors");
  // Stop both motors
  digitalWrite(IN1, LOW);
  digitalWrite(IN2, LOW);
  analogWrite(ENA, 0);
  
  digitalWrite(IN3, LOW);
  digitalWrite(IN4, LOW);
  analogWrite(ENB, 0);
}

void printWiFiStatus() {
  Serial.print("SSID: ");
  Serial.println(WiFi.SSID());
  
  IPAddress ip = WiFi.localIP();
  Serial.print("IP Address: ");
  Serial.println(ip);

  long rssi = WiFi.RSSI();
  Serial.print("Signal strength (RSSI):");
  Serial.print(rssi);
  Serial.println(" dBm");
  
  Serial.println("To control this car, send HTTP requests to:");
  Serial.print("http://");
  Serial.print(ip);
  Serial.println("/command");
  Serial.println("Where command is: forward, backward, left, right, or stop");
}